mod overlay;
mod vk_layer;

use std::{
    fs::{self, File, OpenOptions},
    io::Write,
    path::Path,
};

use overlay::{Overlay, OverlayConfig};

static mut LOG_ENABLED: bool = true;
static mut LOG_FILE: String = String::new();
static mut OVERLAY: Overlay = Overlay::new();

pub(crate) fn logging() -> bool {
    unsafe { LOG_ENABLED }
}

macro_rules! write_log {
    ($msg:expr) => {
        if crate::logging() {
            crate::log($msg);
        }
    };
}

pub(crate) use write_log;

fn get_config(home: &str) -> OverlayConfig {
    let config_path = Path::new(&home).join(".config/rFactorHUD/config.json");

    if config_path.exists() {
        fs::read_to_string(&config_path)
            .map(|s| {
                serde_json::from_str(&s).unwrap_or({
                    write_log!("failed to deserialize config file");
                    OverlayConfig::new()
                })
            })
            .unwrap_or({
                write_log!(format!("failed to open config file: {:?}", config_path));
                OverlayConfig::new()
            })
    } else {
        if let Err(err) = std::fs::create_dir_all(config_path.parent().unwrap()) {
            write_log!(format!("failed to create dirs for config file: {:?}", err));
        }

        let config = OverlayConfig::new();

        match File::create(config_path) {
            Ok(mut file) => match serde_json::to_string_pretty(&config) {
                Ok(conf_str) => {
                    if let Err(err) = file.write_all(conf_str.as_bytes()) {
                        write_log!(format!("failed to write to config file: {:?}", err));
                    }
                }
                Err(err) => {
                    write_log!(format!("failed to serialize config: {:?}", err));
                }
            },
            Err(err) => {
                write_log!(format!("failed to create config file: {:?}", err));
            }
        }

        config
    }
}

pub fn log(msg: impl ToString) {
    assert!(logging());

    if let Ok(mut file) = OpenOptions::new()
        .append(true)
        .create(true)
        .open(unsafe { &LOG_FILE })
    {
        if let Err(_) = file.write_all(format!("{}\n", msg.to_string()).as_bytes()) {}
    }
}

#[cfg(test)]
mod test {
    use crate::get_config;

    #[test]
    fn config() {
        let home = std::env::var("HOME").unwrap();
        get_config(&home);
    }
}
