use std::sync::Arc;

use anyhow::Result;
use rfactor_sm_reader::{rF2VehicleTelemetry, VehicleScoringInfoV01};
use ui::prelude::*;

use crate::overlay::{
    rfactor_data::{DataReceiver, GamePhase},
    UiOverlay,
};

pub struct Watermark {
    gui: Arc<GuiBuilder>,
}

impl Watermark {
    pub fn new(gui_handler: &Arc<GuiHandler>) -> Result<Self> {
        const DESC: &str = include_str!("watermark.xml");

        let gui = GuiBuilder::from_str(gui_handler, DESC)?;

        Ok(Self { gui })
    }
}

impl UiOverlay for Watermark {}

impl DataReceiver for Watermark {
    fn game_phase_change(&mut self, phase: GamePhase) -> Result<()> {
        match phase {
            GamePhase::None => self.gui.enable()?,
            _ => self.gui.disable()?,
        }

        Ok(())
    }

    fn update_for_phase(&self, phase: GamePhase) -> bool {
        match phase {
            _ => false,
        }
    }

    fn scoring_update(
        &mut self,
        _phase: GamePhase,
        _vehicle_scoring: &[VehicleScoringInfoV01],
    ) -> Result<()> {
        Ok(())
    }

    fn telemetry_update(
        &mut self,
        _player_id: Option<i32>,
        _telemetries: &[rF2VehicleTelemetry],
    ) -> Result<()> {
        Ok(())
    }
}
